function love.load()
  scrW = 640
  scrH = 480
  love.window.setTitle("CRT Shader Test")
  love.window.setMode(scrW, scrH, {resizable = false})
  
  screenBuffer = love.graphics.newCanvas(scrW, scrH)
  
  sCRT = love.graphics.newShader [[
    //
    // PUBLIC DOMAIN CRT STYLED SCAN-LINE SHADER
    //
    //   by Timothy Lottes
    //
    // This is more along the style of a really good CGA arcade monitor.
    // With RGB inputs instead of NTSC.
    // The shadow mask example has the mask rotated 90 degrees for less chromatic aberration.
    //
    // Left it unoptimized to show the theory behind the algorithm.
    //
    // It is an example what I personally would want as a display option for pixel art games.
    // Please take and use, change, or whatever.
    //
    
    extern vec3 iResolution;
    extern Image iChannel;    

    // Emulated input resolution.
    #if 0
      // Fix resolution to set amount.
      vec2 res=vec2(320.0/1.0,160.0/1.0);
    #else
      // Optimize for resize.
      vec2 res=iResolution.xy/6.0;
    #endif

    // Hardness of scanline.
    //  -8.0 = soft
    // -16.0 = medium
    number hardScan=-8.0;

    // Hardness of pixels in scanline.
    // -2.0 = soft
    // -4.0 = hard
    number hardPix=-3.0;

    // Display warp.
    // 0.0 = none
    // 1.0/8.0 = extreme
    vec2 warp=vec2(1.0/32.0,1.0/24.0); 

    // Amount of shadow mask.
    number maskDark=0.5;
    number maskLight=1.5;

    //------------------------------------------------------------------------

    // sRGB to Linear.
    // Assuing using sRGB typed textures this should not be needed.
    number ToLinear1(number c){return(c<=0.04045)?c/12.92:pow((c+0.055)/1.055,2.4);}
    vec3 ToLinear(vec3 c){return vec3(ToLinear1(c.r),ToLinear1(c.g),ToLinear1(c.b));}

    // Linear to sRGB.
    // Assuing using sRGB typed textures this should not be needed.
    number ToSrgb1(number c){return(c<0.0031308?c*12.92:1.055*pow(c,0.41666)-0.055);}
    vec3 ToSrgb(vec3 c){return vec3(ToSrgb1(c.r),ToSrgb1(c.g),ToSrgb1(c.b));}

    // Nearest emulated sample given numbering point position and texel offset.
    // Also zero's off screen.
    vec3 Fetch(vec2 pos,vec2 off){
      pos=floor(pos*res+off)/res;
      if(max(abs(pos.x-0.5),abs(pos.y-0.5))>0.5)return vec3(0.0,0.0,0.0);
      return ToLinear(Texel(iChannel,pos.xy,-16.0).rgb);}
    
    // Distance in emulated pixels to nearest texel.
    vec2 Dist(vec2 pos){pos=pos*res;return -((pos-floor(pos))-vec2(0.5));}
    
    // 1D Gaussian.
    number Gaus(number pos,number scale){return exp2(scale*pos*pos);}

    // 3-tap Gaussian filter along horz line.
    vec3 Horz3(vec2 pos,number off){
      vec3 b=Fetch(pos,vec2(-1.0,off));
      vec3 c=Fetch(pos,vec2( 0.0,off));
      vec3 d=Fetch(pos,vec2( 1.0,off));
      number dst=Dist(pos).x;
      // Convert distance to weight.
      number scale=hardPix;
      number wb=Gaus(dst-1.0,scale);
      number wc=Gaus(dst+0.0,scale);
      number wd=Gaus(dst+1.0,scale);
      // Return filtered sample.
      return (b*wb+c*wc+d*wd)/(wb+wc+wd);}

    // 5-tap Gaussian filter along horz line.
    vec3 Horz5(vec2 pos,number off){
      vec3 a=Fetch(pos,vec2(-2.0,off));
      vec3 b=Fetch(pos,vec2(-1.0,off));
      vec3 c=Fetch(pos,vec2( 0.0,off));
      vec3 d=Fetch(pos,vec2( 1.0,off));
      vec3 e=Fetch(pos,vec2( 2.0,off));
      number dst=Dist(pos).x;
      // Convert distance to weight.
      number scale=hardPix;
      number wa=Gaus(dst-2.0,scale);
      number wb=Gaus(dst-1.0,scale);
      number wc=Gaus(dst+0.0,scale);
      number wd=Gaus(dst+1.0,scale);
      number we=Gaus(dst+2.0,scale);
      // Return filtered sample.
      return (a*wa+b*wb+c*wc+d*wd+e*we)/(wa+wb+wc+wd+we);}

    // Return scanline weight.
    number Scan(vec2 pos,number off){
      number dst=Dist(pos).y;
      return Gaus(dst+off,hardScan);}

    // Allow nearest three lines to effect pixel.
    vec3 Tri(vec2 pos){
      vec3 a=Horz3(pos,-1.0);
      vec3 b=Horz5(pos, 0.0);
      vec3 c=Horz3(pos, 1.0);
      number wa=Scan(pos,-1.0);
      number wb=Scan(pos, 0.0);
      number wc=Scan(pos, 1.0);
      return a*wa+b*wb+c*wc;}

    // Distortion of scanlines, and end of screen alpha.
    vec2 Warp(vec2 pos){
      pos=pos*2.0-1.0;    
      pos*=vec2(1.0+(pos.y*pos.y)*warp.x,1.0+(pos.x*pos.x)*warp.y);
      return pos*0.5+0.5;}

    // Shadow mask.
    vec3 Mask(vec2 pos){
      pos.x+=pos.y*3.0;
      vec3 mask=vec3(maskDark,maskDark,maskDark);
      pos.x=fract(pos.x/6.0);
      if(pos.x<0.333)mask.r=maskLight;
      else if(pos.x<0.666)mask.g=maskLight;
      else mask.b=maskLight;
      return mask;}    

    // Draw dividing bars.
    number Bar(number pos,number bar){pos-=bar;return pos*pos<4.0?0.0:1.0;}

    // Entry.
    void mainImage( out vec4 fragColor, in vec2 fragCoord ){
      // Unmodified.
      if(fragCoord.x<iResolution.x*0.333){
        fragColor.rgb=Fetch(fragCoord.xy/iResolution.xy+vec2(0.333,0.0),vec2(0.0,0.0));}
      else{
        vec2 pos=Warp(fragCoord.xy/iResolution.xy+vec2(-0.333,0.0));
        if(fragCoord.x<iResolution.x*0.666){
          hardScan=-12.0;
          maskDark=maskLight=1.0;
          pos=Warp(fragCoord.xy/iResolution.xy);}
        fragColor.rgb=Tri(pos)*Mask(fragCoord.xy);}    
      fragColor.a=1.0;  
      fragColor.rgb*=
        Bar(fragCoord.x,iResolution.x*0.333)*
        Bar(fragCoord.x,iResolution.x*0.666);
      fragColor.rgb=ToSrgb(fragColor.rgb);}
      
    vec4 effect(vec4 color, Image texture, vec2 texture_coords, vec2 pixel_coords){
      vec2 fragCoord = texture_coords * iResolution.xy;
	  mainImage( color, fragCoord );
	  return color;
	} 
  ]]
  
end

function love.update(dt)
end

function love.draw()
  love.graphics.setCanvas(screenBuffer)
  love.graphics.setColor(255, 255, 255)
  love.graphics.circle("fill", scrW / 2, scrH / 2, scrH / 4, 12)
  love.graphics.setCanvas()
    
  sCRT:send("iResolution", {0, scrW, scrH})
  sCRT:send("iChannel", screenBuffer)  
  love.graphics.setShader(sCRT) 
  love.graphics.draw(screenBuffer, 0, 0)  
  love.graphics.setShader()  
end

function love.keypressed(k)
  if k=="escape" then
    love.event.quit()
  end
end